"use strict";


var fs = require("fs");
var chalk = require("chalk");
var path = require("path");
var when = require("when");


var fileCacheById = {};


function getFile(id) {
	return fileCacheById[id];
}


function listDirectories(ofComponents) {
	return when.promise(function (resolve, reject) {
		fs.readdir(ofComponents, function (err, items) {
			return items.map(function (directory) {
				return path.join(ofComponents, directory);
			});
		});
	});
}

function listDirectories(ofComponents) {
	return when.promise(function (resolve, reject) {
		fs.readdir(ofComponents, function (err, items) {
			var directories;

			if (err) {
				return reject(err);
			}

			directories = items.map(function (directory) {
				return path.join(ofComponents, directory);
			});

			resolve(directories);
		});
	});
}



function getPartials(directory) {
	return when.promise(function (resolve, reject) {

		fs.readdir(directory, function (err, items) {

			if (err) {
				return reject(err);
			}

			items = items.filter(function (filename) {
				return /\.hbs$/.test(filename);
			});

			items = items.map(function (partial) {
				return path.join(directory, partial);
			});

			resolve(items);
		});
	});
}

function getPartialId(path) {
	return path.match(/\/([^\/]*\/[^\/]*)$/).pop();
}

function buildAll(listOfFolders) {
	return when.all(listOfFolders.map(buildCache))
		.then(function (result) {
			console.log("partials retrieved");
		});
}

function buildCache(ofComponents) {
	return listDirectories(ofComponents)
		.then(function (directories) {
			return when.all(directories.map(getPartials))
				.then(function mergeToSingleList(lists) {
					lists.forEach(function (list) {
						list.forEach(function (partial) {
							var name = getPartialId(partial);
							fileCacheById[name] = partial;
						});
					});
					return fileCacheById;
				});
		})
		.then(function (cache) {
			console.log(chalk.red("partials loaded", JSON.stringify(cache, null, 4)));
		});
}


module.exports = {

	buildAll: buildAll,
	build: buildCache,
	get: getFile
};

