/* global require, console, module, process */
"use strict";


var fs = require("fs");
var path = require("path");
var when = require("when");
var webpack = require("webpack");
var chalk = require("chalk");
var handlebars = require("handlebars");
var components = require("./components");
var fileCache = require("./fileCache");

var data = {
	"name": "testdata",
	"description": "hardcoded in loader"
};



function getRendered(partial, data) {
	var path = fileCache.get(partial);
	var template = handlebars.compile(fs.readFileSync(path, "utf-8"));

	return template(data);
}

function createComponentHelper(loader) {
	return function componentHelper(partial, directive, json, context) {
		console.log(chalk.blue("component-helper", JSON.stringify(context)));
		console.log(chalk.gray("-- partial: " + partial + " -- directive: " + directive + " -- json: " + json));

		/*
			register script in webpack
		 */
		components.add(directive);


		/*
			register partial in handlebars
		 */
		var html = getRendered(partial, data);
		return html;
	};
}

/*
	called once only
 */
module.exports = function (indexHtmlContents) {
	console.log(chalk.cyan("HANDLEBARS LOADER -- exec"), this);

	var callback = this.async();
	var template = handlebars.compile(indexHtmlContents);

	handlebars.registerHelper("component", createComponentHelper(this));

	fileCache.build(this.options.handlebars.resolve.root[0])
		.then(function saveIndexHtml() {
			var html = template(data);
			fs.writeFileSync(path.join(process.cwd(), "build", "index.html"), html, "utf-8");
		})
		.then(function exportComponents() {
			callback(null, components.toString());
		});
};

